﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.Services;

using Negoci;
using Entitats;

namespace Web
{

    public partial class ListArticulos : System.Web.UI.Page
    {
        #region WebMethods

        //Función para agregar un articulo al carrito
        [WebMethod(EnableSession = true)]
        public static int addCarrito(string codigo)
        {
            int result = 1;
            try
            {
                List<PedidoLin> carrito = listaLin;

                if (listaLin.Count == 0)//El carrito está vacio, miraremos si el usuario está registado y crearemos el pedido.
                {
                    if ((string)(HttpContext.Current.Session["userlogin"]) != "")
                    {
                        PedidoBL newped = new PedidoBL();
                        Num_pedido = newped.InsertPedido((string)(HttpContext.Current.Session["userlogin"]));
                    }
                }

                int existe = 0;

                PedidoBL pedidobl = new PedidoBL();
                ArticuloBL articulobl = new ArticuloBL();
                Articulo art = articulobl.GetArticuloByCodigo(codigo);
                bool envio = false;

                //Buscamos si ya existia en el carrito
                foreach (PedidoLin pl in carrito)
                {
                    if (pl.articulo == codigo)
                    {
                        existe = pl.id_pedido_lin;
                        break;
                    }
                }

                if (existe != 0)
                {
                    //Si existe, aumentamos la cantidad en una unidad
                    result = addCantidad(codigo);
                }
                else
                {
                    PedidoLin nuevo = new PedidoLin();
                    nuevo.articulo = codigo;
                    nuevo.cantidad = 1;
                    nuevo.precio_iva = art.precio_iva;

                    if (Num_pedido != 0)
                        nuevo.pedido = Num_pedido;

                    carrito.Add(nuevo);

                    //Si no existe, creamos una linea nueva
                    //Si es el primer artículo, creamos también la línea del Envío
                    if (carrito.Count == 1)
                    {
                        envio = true;
                        Articulo env = articulobl.GetArticuloByCodigo("ENVIO");
                        PedidoLin linenv = new PedidoLin();
                        linenv.articulo = env.codigo;
                        linenv.cantidad = 1;
                        linenv.precio_iva = env.precio_iva;
                        linenv.pedido = (int)(HttpContext.Current.Session["num_pedido"]);

                        carrito.Add(linenv);
                    }

                    //Si está identificado, lo guardamos en bbdd
                    if ((string)(HttpContext.Current.Session["userlogin"]) != "")
                    {
                        nuevo.id_pedido_lin = pedidobl.InsertPedidoLin(nuevo.pedido, nuevo.articulo, nuevo.cantidad, nuevo.precio_iva, nuevo.descuento, envio);
                    }

                    //Actualizamos la lista de la sesión
                    listaLin = carrito;
                }
            }
            catch
            {
                result = 0;
            }
            return result;

        }

        //Función para aumentar la cantidad de un artículo
        [WebMethod(EnableSession = true)]
        public static int addCantidad(string codigo)
        {
            int result = 1;
            try
            {
                List<PedidoLin> carrito = listaLin;
                int id_pedido_lin = 0;
                int cantidad_nueva = 0;

                PedidoBL pedidobl = new PedidoBL();
                ArticuloBL articulobl = new ArticuloBL();

                //Buscamos el artículo en cuestión para añadirle una unidad más
                foreach (PedidoLin pl in carrito)
                {
                    if (pl.articulo == codigo)
                    {
                        id_pedido_lin = pl.id_pedido_lin;

                        cantidad_nueva = pl.cantidad + 1;
                        pl.cantidad = cantidad_nueva;
                        break;
                    }
                }

                //Si está identificado, lo guardamos en bbdd
                if ((string)(HttpContext.Current.Session["userlogin"]) != "")
                {
                    if (id_pedido_lin != 0)
                        pedidobl.ActualizarCantidadPedidoLin(id_pedido_lin, cantidad_nueva);
                }

                //Actualizamos la lista de la sesión
                listaLin = carrito;
            }
            catch
            {
                result = 0;
            }
            return result;
        }

        //Función que resta una unidad a la cantidad de un artículo del carrito
        [WebMethod(EnableSession = true)]
        public static int restarCantidad(string codigo)
        {
            int result = 1;
            bool envio = false;
            try
            {
                List<PedidoLin> carrito = listaLin;

                int existe = 0;
                int cantidad_nueva = 0;

                PedidoBL pedidobl = new PedidoBL();
                ArticuloBL articulobl = new ArticuloBL();

                foreach (PedidoLin pl in carrito)
                {
                    if (pl.articulo == codigo)
                    {
                        existe = pl.id_pedido_lin;

                        cantidad_nueva = pl.cantidad - 1;
                        if (cantidad_nueva != 0)
                            pl.cantidad = cantidad_nueva;
                        else
                        {
                            //Si borramos el ultimo articulo, quitamos tambien el envío
                            if (carrito.Count > 2)
                            {
                                carrito.Remove(pl);
                            }
                            else
                            {
                                envio = true;
                                carrito = new List<PedidoLin>();
                            }
                        }

                        break;
                    }
                }

                //Si está identificado, lo guardamos en bbdd
                if ((string)(HttpContext.Current.Session["userlogin"]) != "")
                {
                    if (existe != 0)
                    {
                        //Si la cantidad actual es diferente de zero, borramos la línea
                        if (cantidad_nueva != 0)
                            pedidobl.ActualizarCantidadPedidoLin(existe, cantidad_nueva);
                        else
                        {
                            //Si está registrado y tenia un pedido pendiente, al quedarse sin líneas, lo borramos
                            Pedido ped = pedidobl.GetPedidoPdteByUsuario((string)(HttpContext.Current.Session["userlogin"]));
                            if (ped != null)
                                pedidobl.DeletePedido(ped.numero);
                            else
                                pedidobl.EliminarPedidoLin(existe, envio);
                        }
                    }
                }

                //Actualizamos la lista de la sesión
                listaLin = carrito;
            }
            catch
            {
                result = 0;
            }
            return result;
        }

        #endregion

        #region Propiedades de la Clase

        protected static List<PedidoLin> listaLin
        {
            get { return (List<PedidoLin>)HttpContext.Current.Session["listalin"]; }
            set
            {
                HttpContext.Current.Session["listalin"] = value;
            }
        }

        protected static int Num_pedido
        {
            get { return (int)HttpContext.Current.Session["num_pedido"]; }
            set { HttpContext.Current.Session["num_pedido"] = value; }
        }

        #endregion

        protected void Page_Load(object sender, EventArgs e)
        {
            //La primera vez
            if (!IsPostBack)
            {
                //Actualizar carrito
                cargarCarrito();
            }

            //Cargar Menú según las categorías
            cargarMenu();

            montarTablaArticulos();
        }

        //Funcion para Cargar el Menú
        private void cargarMenu()
        {
            CategoriaBL catBL = new CategoriaBL();
            List<Categoria> categorias = new List<Categoria>();
            categorias = catBL.GetCategoriasNoEnvio();

            foreach (Categoria c in categorias)
            {
                MenuItem mi = new MenuItem();
                mi.NavigateUrl = "~/ListArticulos.aspx?cat=" + c.codigo;
                mi.Text = c.descripcion;
                menu.Items.Add(mi);
            }
        }

        //Funcion para Cargar el Carrito
        private void cargarCarrito()
        {
            string userlogin = Session["userlogin"].ToString();

            if (userlogin != "")
            {
                PedidoBL pedidobl = new PedidoBL();

                if (listaLin.Count == 0)
                {
                    listaLin = pedidobl.GetPedidoLinPdteByUsuario(Session["userlogin"].ToString());
                    Num_pedido = pedidobl.GetNumPdteByUsuario(Session["userlogin"].ToString());
                }
            }
            else
            {
                if (listaLin.Count == 0)
                    listaLin = new List<PedidoLin>();
                Num_pedido = -1;
            }

            gvPedidoLin.DataSource = listaLin;
            gvPedidoLin.DataBind();
        }

        //Funcion para Cargar el Total del Carrito
        protected void cargarTotal()
        {
            decimal total = 0;

            if (listaLin.Count > 0)
            {
                lblNoHayLin.Visible = false;
                lblTotal.Visible = true;
                btnComprar.Visible = true;

                foreach (PedidoLin ped_lin in listaLin)
                {
                    total += (ped_lin.precio_iva * ped_lin.cantidad);
                }

                lblTotal.Text = "TOTAL Compra " + total.ToString() + "€";
            }
            else
            {
                lblNoHayLin.Visible = true;
                lblTotal.Visible = false;
                btnComprar.Visible = false;
            }
        }

        protected string btnAddCant(Object articulo)
        {
            string cadena = "";
            if (articulo.ToString() != "ENVIO")
                cadena = @"<a onclick=""btnAddCantClick('" + articulo + @"')""><img alt='Añadir' src='../icons/add.png' /></a>";

            return cadena;
        }

        protected string btnElimCant(Object articulo)
        {
            string cadena = "";
            if (articulo.ToString() != "ENVIO")
                cadena = @"<a onclick=""btnEliminarCantClick('" + articulo + @"')""><img alt='Eliminar' src='../icons/delete.png' /></a>";

            return cadena;
        }

        protected void montarTablaArticulos()
        {
            string categoria = Request.Params["cat"];
            string tabla = "";

            tabla = @"<table border=""1px"" width=""100%"">";

            tabla += "<tr>";

            int cont = 0;

            List<Articulo> la = new List<Articulo>();

            ArticuloBL abl = new ArticuloBL();
            la = abl.GetArticulosByCategoria(categoria);

            foreach (Articulo a in la)
            {

                string stock = "";
                string carrito = "";

                if (a.stock > 0)
                {
                    stock = @"<p style=""font-weight: bold; color: #008000"">En stock</p>";
                    carrito = string.Format(@"<img src=""..\icons\cart_add.png"" onclick=""btnAddCart('{0}')"" /> </td>", a.codigo);
                }
                else
                {
                    stock = @"<p style=""font-weight: bold; color: #FF0000"">No disponible</p>";
                    carrito = @"</td>";
                }


                cont++;
                if (cont == 4)
                {

                    tabla += @"<td align=""center"" width=""25%"">";
                    tabla += @"<table Width=""33%"">";
                    tabla += @"<tr align=""center"">";
                    tabla += string.Format(@"<td> <img src=""{0}"" width=""100px"" onclick=""btnDetalleArticulo('{1}')"" /></td>", a.ruta_foto, a.codigo);
                    tabla += "</tr>";
                    tabla += @"<tr align=""center"">";
                    tabla += string.Format(@"<td align=""center""> {0} </td>", a.descripcion);
                    tabla += "</tr>";
                    tabla += @"<tr align=""center"">";

                    tabla += string.Format(@"<td align=""center""> {0}{1} ", a.precio_iva, "€");
                    tabla += carrito;

                    tabla += "</tr>";

                    tabla += @"<tr align=""center"">";
                    tabla += string.Format(@"<td align=""center""> {0} </td>", stock);
                    tabla += "</tr>";

                    tabla += "</table>";


                    tabla += "</td>";
                    tabla += "</tr>";
                    tabla += "<tr>";
                    cont = 0;
                }
                else
                {
                    tabla += @"<td align=""center"" width=""25%"">";
                    tabla += @"<table Width=""33%"">";
                    tabla += @"<tr align=""center"">";
                    tabla += string.Format(@"<td> <img src=""{0}"" width=""100px"" onclick=""btnDetalleArticulo('{1}')"" /></td>", a.ruta_foto, a.codigo);
                    tabla += "</tr>";
                    tabla += @"<tr align=""center"">";
                    tabla += string.Format(@"<td align=""center""> {0} </td>", a.descripcion);
                    tabla += "</tr>";
                    tabla += @"<tr align=""center"">";

                    tabla += string.Format(@"<td align=""center""> {0}{1} ", a.precio_iva, "€");
                    tabla += carrito;

                    tabla += "</tr>";

                    tabla += @"<tr align=""center"">";
                    tabla += string.Format(@"<td align=""center""> {0} </td>", stock);
                    tabla += "</tr>";

                    tabla += "</table>";

                    tabla += "</td>";
                }


            }
            tabla += "</tr>";
            tabla += "</table>";

            litArticles.Text = tabla;

        }

        protected void UpdatePanel1_Load(object sender, EventArgs e)
        {
            gvPedidoLin.DataSource = listaLin;
            gvPedidoLin.DataBind();
            cargarTotal();
        }

        protected void btnComprar_Click(object sender, EventArgs e)
        {
            string login = Session["userlogin"].ToString();

            //Si está identificado
            if (login != "")
            {
                //Comprobar datos de cliente
                UsuarioBL usuarioBL = new UsuarioBL();

                if (usuarioBL.isCliente(login))
                {
                    //Si ya tiene los datos personales, hacemos la factura
                    PedidoBL pedidobl = new PedidoBL();
                    pedidobl.FacturarPedido(login);
                }
                else
                {
                    //Sino, lo redirigimos para que introduzca sus datos personales
                    Response.Redirect("~/MiCuenta.aspx?op=new&comp=1");
                }
            }
            else
            {
                //Redireccionar para que se registre o identifique
                Session["comp"] = 1;
                Response.Redirect("~/Account/Login.aspx");
            }

            listaLin = new List<PedidoLin>();

            cargarCarrito();
            cargarTotal();
        }
    }
}